%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input, need to be in the workspace before run this 
%
%
%Mlink - pairwise similarity matrix as in (11)
%Frame_start - a vector containing the start frame number of tracklets
%Frame_finish - a vector containing the finish frame number of tracklets
%
%Data - cells that contain your tracklet data. In our case, we use vectors
%of size ((3+4*T)*1), where T is the length of one tracklet. Each term of Data should
%read in the text file containing tracklet information in order. See
%sampletracklet.txt for an example of a tracklet. Line 1 is the start
%frame, Line 2 the finish frame. Line 3 not used. Then every four numbers
%indicate the x position, y position, rectangle width and height for each
%frame on the image plane.
%
%%%%%%output
%F_Score - All the final scores for all runs     
%Track_collection - Keep all the corresponding final output Tracks for all runs
%ScoreforK - Lowest score for each K
%TrackforK - The correponding output tracks for each K
%
% Copyright 2012, Zhen Qin and Christian R. Shelton
% Provided free for non-commercial use
% WITHOUT ANY WARRANTY -- see Readme.txt

function [F_Score, Track_collection, ScoreforK TrackforK] = MainOptimize(Mlink,Frame_start,Frame_finish,Data)

clc
%%%%%%%%%%%%%%%Need to set
%NumofTr - Number of Tracklets
NumofTr = 10;
%alpha in (3), need to acquire from training
WEIGHT=0.12;
%Augment value described in (14), acquire from training
thresfinish=30;
%N of line 2 in Alg.1
RUNTIME=10;
%Width and height of the image
Width = 640;
Height = 480;
%K_m of line 1 in Alg.1
K_m = 2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%augment the matrix
Mlink_new=zeros(NumofTr*2,NumofTr*2);
Mlink_new(NumofTr+1:NumofTr*2,:)=Inf;
for i=NumofTr+1:NumofTr*2
    Mlink_new(i-NumofTr,i)=thresfinish;
end

StartFrame=min(Frame_start);   %in order to generate centres all along the video, record its start and finish point
FinishFrame=max(Frame_finish);
Tracklet_cell=cell(NumofTr,1);  %use cell to store all tracklet info
     for i=1:NumofTr
         tmpdata=cell2mat(Data(i,1));
         Tracklet_cell(i)={tmpdata};
     end

%Keep all the final scores for all runs (line 16 in Alg.1)
F_Score=[];    
%Keep all the corresponding final output Tracks for all runs
Track_collection=[];
%Lowest score for each K
ScoreforK=[];
%The corresponding output Tracks for each K
TrackforK=[];
%loop for line 1 in Alg.1     
for K=2:K_m

GTracklets=cell(K,1); %Grouped tracklets
FinalS=100000000;    
FinalG=0;
    %loop for line 2 in Alg.1
    for run_time=1:RUNTIME
        Dis=[];
        run_time
        %\mu in the paper
        u=zeros(NumofTr,NumofTr,K); 
        %description of psi and phi are inversed in the paper
        psi=zeros(NumofTr, NumofTr); 
        phi=zeros(NumofTr,K); 
        
        lastphi=zeros(NumofTr,K); %for the last center generation
        randt=randperm(NumofTr);
        GTracklets2=cell(K,1);
        
        %random initial group assignment
        for numofK=1:K
            tt=randt(numofK);
            GTracklets(numofK)={tt};          
        end

        for m=(K+1):NumofTr
            seed=floor(rand(1)*K)+1;%assign
            tmp1=cell2mat(GTracklets(seed));
            tmp2=randt(m);
            tmp1=[tmp1 tmp2];
            GTracklets(seed)={tmp1};
        end
        %Number of frames in the time window
        Rows=FinishFrame-StartFrame+1;
        
        Centres = Centre_Gen_Town(GTracklets,Rows,K,Data,FinishFrame,StartFrame,NumofTr,Tracklet_cell,Width,Height); %initial centres
        %perform optimization using line-search
        [u,fval,psi,phi,Centre_new,distmat_new] = trackoptimize_new(u,Rows,K,Data,FinishFrame,StartFrame,NumofTr,Tracklet_cell,phi,psi,Mlink,thresfinish,Centres,WEIGHT,Width,Height);
    
       
        fval=-fval
        u=reshape(u,[NumofTr NumofTr K]);
        for i=1:NumofTr
            for j=1:NumofTr
                tmpk=0;
                for k=1:K
                    tmpk=tmpk+u(i,j,k)*(phi(i,k)-phi(j,k));
                end
            Mlink_new(i,j)=Mlink(i,j)+tmpk;
            end
        end
    


        %bipartite matching
        [Matching_new,Cost_new]=munkres(Mlink_new);
        final_matching=Matching_new;
        %[GroundTruthNum,CareNum, CareVector]=newGTfor38(final_matching,NumofTr);%%%%%%%%%%%%%%%%!!!!!!!!!!!!!

        %get the linked tracks (in terms of the tracklet numbers), the last
        %number of each track should be bigger than NumofTr, meaning it is
        %terminated
        [row,col]=find(final_matching==1);
    
        S_Label=[];
        for i=1:NumofTr %only start at exsiting tracklets
            if(Find(col,i)==0) %can not find in columns
                S_Label=[S_Label i]; %so it is the starting label of one track
            end
        end
        Target_Num1=size(S_Label,2);
        Tracks1=cell(Target_Num1,1);

        Sum1=0;
        bigphi=zeros(Target_Num1,K);
        for i=1:Target_Num1
            
            tracks1=[];
            temp = S_Label(i);
            tracks1=temp;
            while(Find(row,temp)==1) %in row, so not the last tracklet in the track
                tar=find(row==temp);
                tracks1 = [tracks1 col(tar)];
                temp = col(tar);
            end
        Tracks1(i,1)={tracks1}; 
        Track_collection=[Track_collection,{Tracks1}];

%%%%%%%%% perform section 4.5
        num1=size(tracks1,2)-1;
        member=tracks1(1:num1);
        for k=1:K
            dis=0;
            for j=1:num1            
                dis=dis+mycalcdist_new(Tracklet_cell(tracks1(j)),Centre_new(:,2*k-1),Centre_new(:,2*k),StartFrame,Width,Height);  
            end
            Dis(k)=dis;
        end
        [c,nnn]=min(Dis);%shortest distance of each tracklet to the centres
 
        lastphi(member,nnn)=1;
        tmptrack=cell2mat(GTracklets2(nnn));
        tmptrack=[tmptrack member];
        GTracklets2(nnn)={tmptrack};
               
        bigphi(i,nnn)=1;
    
        end
        %get the last centers
        new_centers = Centre_Gen_Town(GTracklets2,Rows,K,Data,FinishFrame,StartFrame,NumofTr,Tracklet_cell,Width,Height);
        Dis=0;
        for i=1:Target_Num1
            k=find(bigphi(i,:)==1);
            member=cell2mat(Tracks1(i));
            nn=size(member,2)-1;
            for j=1:nn
                Dis=Dis+mycalcdist_new(Tracklet_cell(member(j)),new_centers(:,2*k-1),new_centers(:,2*k),StartFrame,Width,Height);   
            end
        end
        %quality of social group
        Sum1=Dis;

        %%%%%%%%%%%%%%Quality of tracklet linking
        
        sum1_2=size(Tracks1,1)*thresfinish;
        [Row_final,Col_final]=find(psi==1);
        sum1_1=sum(diag(Mlink(Row_final,Col_final)));
        sum1=sum1_1+sum1_2;
                       
        %%%%%%%%%%%%%%Final Score
                
        Final_Score=sum1+WEIGHT*Sum1
        % Keep the one with minimum F_Score for each K
        if FinalS>Final_Score
            FinalS=Final_Score;
            TTT=Tracks1;
        end
        
        F_Score=[F_Score;Final_Score];

    end
    
ScoreforK=[ScoreforK,FinalS];
%The corresponding output Tracks for each K
TrackforK=[TrackforK,TTT];
%You can perform linear pernalty for them
end


 



